<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\Product;
use App\Models\CartItem;
use Illuminate\Support\Facades\DB;
use App\Models\User;
use App\Models\Checkout;
use App\Models\Order;
use Illuminate\Support\Facades\Log;
class CartController extends Controller
{
    // Exibir itens do carrinho
    public function index()
    {
        // Verifica se o usuário está logado
        if (!Auth::check()) {
            return redirect()->route('login')->with('error', 'Você precisa estar logado para acessar o carrinho.');
        }

        $userId = Auth::id();

        // Consulta para obter os itens do carrinho com status 'in_cart'
        $carrinho = DB::table('cart_items')
        ->join('products', 'cart_items.product_id', '=', 'products.id_product')
        ->join('users', 'cart_items.user_id', '=', 'users.id')
        ->select(
            'cart_items.*',
            'products.name',
            'products.price',
            'products.is_promotion',
            'products.discount_percentage',
            'users.name as user_name'
        )
        ->where('cart_items.user_id', auth()->id())
        ->where('cart_items.status', 'in_cart')
        ->get();

        // Obtenha os dados da conta bancária
        $bankAccounts = DB::table('bank_accounts')->get();
        $cartItemCounts = CartItem::where('user_id', $userId)
        ->where('status', 'in_cart')
        ->sum('quantity');
        // Obtenha as chaves Pix
        $pixKeys = DB::table('pix')->get();

        // Retorna a view com os itens do carrinho
        return view('cart.index', compact('carrinho', 'bankAccounts', 'pixKeys','cartItemCounts'));
    }


    public function checkout()
    {
        // Obtenha o código de pedido atual do usuário logado
        $userId = auth()->id();
        $orderCode = CartItem::where('user_id', $userId)
            ->where('status', 'in_cart')
            ->groupBy('order_code') // Garantir que o código de pedido seja único
            ->pluck('order_code') // Pega o código de pedido
            ->first(); // Pegue o primeiro código encontrado

        // Verificar se o código do pedido foi encontrado
        if (!$orderCode) {
            // Se o código de pedido não for encontrado, redirecione ou exiba um erro
            return redirect()->route('cart.index')->with('error', 'Código de pedido não encontrado.');
        }

        // Obtenha todas as contas bancárias do sistema
        $bankAccounts = DB::table('bank_accounts')->get(); // Certifique-se de que a tabela 'bank_accounts' existe no banco

        // Obtenha todas as chaves Pix do sistema
        $pixKeys = DB::table('pix')->get(); // Certifique-se de que a tabela 'pix' existe no banco

        // Retorne a view com os dados do sistema
        return view('cart.checkout', [
            'bankAccounts' => $bankAccounts,
            'pixKeys' => $pixKeys,
            'orderCode' => $orderCode, // Passando o código do pedido para a view
        ]);
    }



    // Processar o checkout após o envio do formulário
    public function processCheckout(Request $request)
    {
        // Lógica para processar o pedido, salvar no banco, etc.
        $paymentMethod = $request->input('payment_method');

        if ($paymentMethod == 'bank_account') {
            // Processar dados de conta bancária
        } elseif ($paymentMethod == 'pix') {
            // Processar dados de Pix
        }

        // Redirecionar para uma página de sucesso ou para o WhatsApp
        return redirect()->route('order.success');
    }
    // Armazena os dados do checkout

// app/Http/Controllers/CheckoutController.php
public function store(Request $request)
{
    try {
        // Validar os dados do formulário
        $validated = $request->validate([
            'order_code' => 'required|string|max:255',
            'address' => 'required|string|max:255',
            'city' => 'required|string|max:255',
            'postal_code' => 'required|string|max:9',
            'state' => 'required|string|max:2',
            'contact' => 'required|string|max:255',
            'phone_number' => 'required|string|max:15',
            'payment_method' => 'required|string',
            'bank_account_id' => 'nullable|exists:bank_accounts,id',
            // Ignorar a validação do pix_key_id caso sempre seja "1"
        ]);

        // Gerar um código de pedido único


        // Criar o pedido no banco de dados
        $checkout = Checkout::create([

            'address' => $validated['address'],
            'city' => $validated['city'],
            'postal_code' => $validated['postal_code'],
            'state' => $validated['state'],
            'contact' => $validated['contact'],
            'phone_number' => $validated['phone_number'],
            'payment_method' => $validated['payment_method'],
            'bank_account_id' => $validated['payment_method'] === 'bank_account' ? $validated['bank_account_id'] : null,
            'pix_key_id' => $validated['payment_method'] === 'pix' ? 1 : null, // Define como 1 quando o método de pagamento for Pix
            'order_code' => $validated['order_code'],
            'user_id' => Auth::id(), // Associa o pedido ao usuário logado
        ]);

        // Atualizar os itens no carrinho para 'completed' após o pedido ser feito
        CartItem::where('user_id', Auth::id())
            ->where('status', 'in_cart')
            ->update(['status' => 'completed']);

        // Redirecionar o usuário com uma mensagem de sucesso
        return redirect()->route('client.area')->with('success', 'Pedido realizado com sucesso!');
    } catch (\Exception $e) {
        // Registrar o erro no log
        Log::error($e->getMessage());

        // Retornar para o formulário com a mensagem de erro
        return redirect()->back()->with('error', 'Erro ao salvar: ' . $e->getMessage())->withInput();
    }
}






    // Método para redirecionar para o WhatsApp
    private function redirectToWhatsApp($checkout)
    {
        $message = "Novo pedido:\n";
        $message .= "Endereço: {$checkout->address}, {$checkout->city}, {$checkout->state}, {$checkout->postal_code}\n";
        $message .= "Forma de pagamento: {$checkout->payment_method}\n";

        if ($checkout->payment_method == 'bank_account') {
            $message .= "Dados Bancários: {$checkout->bank_account_number} - {$checkout->bank_account_agency} - {$checkout->bank_account_owner}\n";
        } elseif ($checkout->payment_method == 'pix') {
            $message .= "Chave Pix: {$checkout->pix_key}\n";
        }

        $whatsappUrl = "https://wa.me/5581XXXXXXX?text=" . urlencode($message);

        // Redireciona para a conversa do WhatsApp
        return redirect()->to($whatsappUrl);
    }





    // Adicionar item ao carrinho
    public function add(Product $product)
    {
        if (!auth()->check()) {
            Log::warning('Usuário não está autenticado.');
            return redirect()->route('login')->with('error', 'Você precisa estar logado para adicionar produtos ao carrinho.');
        }

        $userId = auth()->id();
        Log::info('Usuário autenticado', ['user_id' => $userId]);

        try {
            // Verificar se o usuário já tem itens no carrinho com o status 'in_cart'
            $existingCartItems = CartItem::where('user_id', $userId)
                ->where('status', 'in_cart')
                ->exists();

            // Gerar um código de pedido se for o primeiro item no carrinho
            if (!$existingCartItems) {
                $orderCode = $this->generateUniqueOrderCode();
                Log::info('Novo código de pedido gerado.', ['order_code' => $orderCode]);
            } else {
                // Recuperar o código de pedido existente
                $orderCode = CartItem::where('user_id', $userId)
                    ->where('status', 'in_cart')
                    ->first()
                    ->order_code;
            }

            // Calcular o preço final (considerando promoção)
            $finalPrice = $product->is_promotion && $product->discount_percentage > 0
                ? $product->price * (1 - ($product->discount_percentage / 100))
                : $product->price;

            // Verificar se o produto já existe no carrinho
            $cartItem = CartItem::where('user_id', $userId)
                ->where('product_id', $product->id_product)
                ->where('status', 'in_cart')
                ->first();

            if ($cartItem) {
                // Garante que o order_code esteja presente
                if (is_null($cartItem->order_code)) {
                    $cartItem->order_code = $orderCode ?? $this->generateUniqueOrderCode();
                }

                $cartItem->quantity += 1;

                // Atualiza o preço e o nome do produto
                $cartItem->price = $finalPrice;
                $cartItem->name = $product->name;

                $cartItem->save();

                Log::info('Quantidade atualizada no carrinho.', ['cart_item' => $cartItem]);
            } else {
                // Adicionar o produto ao carrinho com o preço e o nome do produto
                CartItem::create([
                    'user_id' => $userId,
                    'product_id' => $product->id_product,
                    'name' => $product->name,
                    'quantity' => 1,
                    'price' => $finalPrice,
                    'status' => 'in_cart',
                    'order_code' => $orderCode,
                ]);

                Log::info('Novo item adicionado ao carrinho.', ['product_id' => $product->id_product]);
            }

            // Calcular o total do pedido
            $orderTotal = CartItem::where('user_id', $userId)
                ->where('status', 'in_cart')
                ->sum(DB::raw('quantity * price'));

            // Atualizar o total do pedido no banco de dados (se existir um campo correspondente)
            Order::updateOrCreate(
                ['order_code' => $orderCode],
                ['total_amount' => $orderTotal, 'user_id' => $userId]
            );

            Log::info('Total do pedido atualizado.', ['order_code' => $orderCode, 'total_amount' => $orderTotal]);

            return redirect()->route('cart.index')->with('success', 'Produto adicionado ao carrinho com sucesso.');
        } catch (\Exception $e) {
            Log::error('Erro ao adicionar produto ao carrinho.', [
                'message' => $e->getMessage(),
                'user_id' => $userId,
                'product_id' => $product->id_product,
            ]);

            return redirect()->back()->with('error', 'Ocorreu um erro ao adicionar o produto ao carrinho.');
        }
    }




    /**
     * Gera um código único de pedido numérico.
     */
    private function generateUniqueOrderCode()
    {
        do {
            // Gera um número aleatório de 6 dígitos
            $orderCode = mt_rand(100000, 999999);

            // Verifica se o código já existe no banco de dados
            $exists = CartItem::where('order_code', $orderCode)->exists();

            Log::info('Verificação de código único.', [
                'order_code' => $orderCode,
                'exists' => $exists
            ]);
        } while ($exists);

        return $orderCode;
    }

    /**
     * Gera um código único de pedido numérico.
     */



    // Remover item do carrinhopublic function remove($id)
    public function remove($id)
{
    try {
        $cartItem = CartItem::findOrFail($id); // Buscar o item com o ID
        $cartItem->delete();  // Deletar o item

        return redirect()->route('cart.index')->with('success', 'Item removido com sucesso.');
    } catch (\Exception $e) {
        return redirect()->route('cart.index')->with('error', 'Erro ao remover item: ' . $e->getMessage());
    }
}



    // Atualizar itens do carrinho
    public function update(Request $request)
    {
        $validated = $request->validate([
            'quantity.*' => 'required|integer|min:1',
        ]);

        // Atualiza os itens do carrinho
        foreach ($request->input('quantity') as $itemId => $quantity) {
            $cartItem = CartItem::find($itemId);

            if ($cartItem && $cartItem->user_id === auth()->id()) {
                $cartItem->update(['quantity' => $quantity]);
            }
        }

        return redirect()->route('cart.index')->with('success', 'Carrinho atualizado com sucesso!');
    }

    public function finalizeOrder(Request $request)
{
    // Validação dos campos obrigatórios
    $validatedData = $request->validate([
        'address' => 'required|string|max:255',
        'city' => 'required|string|max:255',
        'postal_code' => 'required|string|max:20',
        'state' => 'required|string|max:50',
        'payment_method' => 'required|in:bank_account,pix',
    ]);

    // Criação do pedido
    $checkout = new Checkout();
    $checkout->user_id = Auth::id();
    $checkout->address = $validatedData['address'];
    $checkout->city = $validatedData['city'];
    $checkout->postal_code = $validatedData['postal_code'];
    $checkout->state = $validatedData['state'];
    $checkout->payment_method = $validatedData['payment_method'];
    $checkout->save();

    // Atualizar status dos itens no carrinho
    CartItem::where('user_id', Auth::id())
        ->where('status', 'in_cart')
        ->update(['status' => 'completed']);

    return redirect()->route('client.area')->with('success', 'Pedido finalizado com sucesso!');
}
public function clientArea()
{
    $userId = Auth::id();
    $userRole = Auth::user()->role; // Assumindo que você tem um campo 'role' para definir o papel do usuário

    // Verifica se o usuário é admin
    if ($userRole === 'admin') {
        // Administradores veem todos os pedidos
        $checkouts = Checkout::with(['items' => function ($query) {
            $query->whereHas('product'); // Apenas itens que têm produtos associados
        }, 'items.product'])->get();
    } else {
        // Usuários comuns veem apenas seus próprios pedidos
        $checkouts = Checkout::with(['items' => function ($query) {
            $query->whereHas('product'); // Apenas itens que têm produtos associados
        }, 'items.product'])
        ->where('user_id', $userId)
        ->get();
    }

    $orderStatuses = [
        'cancelado' => 'Cancelado',
        'preparando_para_envio' => 'Preparando para Envio',
        'postado_no_correios' => 'Postado no Correios',
        'recebido' => 'Recebido',
        'aguardando_pagamento' => 'Aguardando Pagamento',
    ];

    return view('client.area', compact('checkouts', 'orderStatuses'));
}





    public function updateStatus(Request $request, $id)
    {
        $validated = $request->validate([
            'status' => 'required|string',
            'tracking_code' => 'nullable|string|max:255', // Opcional, máximo 255 caracteres
        ]);

        // Buscar o pedido pelo ID
        $checkout = Checkout::findOrFail($id);

        // Atualizar o status
        $checkout->status = $validated['status'];

        // Adicionar o código de rastreamento se necessário
        if ($validated['status'] === 'postado_no_correios') {
            $checkout->tracking_code = $validated['tracking_code'];
        } else {
            $checkout->tracking_code = null; // Limpar o código se o status mudar
        }

        $checkout->save();

        return redirect()->back()->with('success', 'Status do pedido atualizado com sucesso!');
    }

    public function uploadProof(Request $request, $id)
    {
        $validated = $request->validate([
            'payment_proof' => 'required|file|mimes:pdf,jpg,jpeg,png|max:2048', // Arquivo de até 2MB
        ]);

        // Buscar o pedido pelo ID
        $checkout = Checkout::findOrFail($id);

        // Fazer o upload do comprovante
        if ($request->hasFile('payment_proof')) {
            $path = $request->file('payment_proof')->store('proofs', 'public'); // Salvar na pasta "proofs" no disco "public"
            $checkout->payment_proof = $path; // Atualizar no banco de dados
            $checkout->save();
        }

        return redirect()->back()->with('success', 'Comprovante enviado com sucesso!');
    }

}
